/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd All Rights Reserved
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
#pragma once

#include <functional>
#include <utility>
#include <type_traits>

#include "common/macros.h"

namespace Csr {

class Mainloop {
public:
	enum class Event : uint32_t {
		NONE  = 0,
		READ  = 1 << 0,
		WRITE = 1 << 1,
		CLOSE = 1 << 2
	};

	using Callback = std::function<void(Event event)>;

	Mainloop() : m_domainSourceNum(0) {}
	virtual ~Mainloop() {}

	Mainloop(const Mainloop &) = delete;
	Mainloop &operator=(const Mainloop &) = delete;
	Mainloop(Mainloop &&) = delete;
	Mainloop &operator=(Mainloop &&) = delete;

	// timeout unit: seconds
	// if timeout is negative value, no timeout on idle.
	virtual void run(int timeout) = 0;

	virtual void addEventSource(int fd, Event event, Callback &&callback) = 0;
	virtual void removeEventSource(int fd) = 0;

	void addDomainEventSource(int fd, Event event, Callback &&callback)
	{
		++this->m_domainSourceNum;
		this->addEventSource(fd, event, std::move(callback));
	}

protected:
	size_t m_domainSourceNum;
};

template<typename T> using Underlying = typename std::underlying_type<T>::type;
template<typename T>
constexpr Underlying<T> underlying(T t) { return Underlying<T>(t); }

inline constexpr Mainloop::Event operator&(Mainloop::Event e1, Mainloop::Event e2)
{
	return Mainloop::Event(underlying(e1) & underlying(e2));
}

inline Mainloop::Event &operator&=(Mainloop::Event &e1, Mainloop::Event e2)
{
	return e1 = e1 & e2;
}

inline constexpr Mainloop::Event operator|(Mainloop::Event e1, Mainloop::Event e2)
{
	return Mainloop::Event(underlying(e1) | underlying(e2));
}

inline Mainloop::Event &operator|=(Mainloop::Event &e1, Mainloop::Event e2)
{
	return e1 = e1 | e2;
}

inline constexpr Mainloop::Event operator~(Mainloop::Event e)
{
	return Mainloop::Event(~underlying(e));
}

} // namespace Csr
