/*
 *  Copyright (c) 2016 Samsung Electronics Co., Ltd All Rights Reserved
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License
 */
/*
 * @file        native-mainloop.cpp
 * @author      Kyungwook Tak (k.tak@samsung.com)
 * @version     1.0
 * @brief       Mainloop of csr-server with epoll
 */
#include "common/native-mainloop.h"

#include <system_error>
#include <sys/epoll.h>
#include <unistd.h>

#include "common/audit/logger.h"
#include "common/exception.h"

namespace Csr {

uint32_t NativeMainloop::convertFlags(Mainloop::Event events)
{
	uint32_t flags = 0;

	if ((events & Mainloop::Event::READ) != Mainloop::Event::NONE)
		flags |= EPOLLIN;
	if ((events & Mainloop::Event::WRITE) != Mainloop::Event::NONE)
		flags |= EPOLLOUT;
	if ((events & Mainloop::Event::CLOSE) != Mainloop::Event::NONE)
		flags |= (EPOLLHUP | EPOLLRDHUP);

	return flags;
}

Mainloop::Event NativeMainloop::convertFlags(uint32_t events)
{
	Mainloop::Event flags = Mainloop::Event::NONE;

	if (events & EPOLLIN)
		flags |= Mainloop::Event::READ;
	if (events & EPOLLOUT)
		flags |= Mainloop::Event::WRITE;
	if (events & (EPOLLHUP | EPOLLRDHUP))
		flags |= Mainloop::Event::CLOSE;

	return flags;
}

NativeMainloop::NativeMainloop() :
	m_isTimedOut(false),
	m_pollfd(::epoll_create1(EPOLL_CLOEXEC))
{
	if (this->m_pollfd == -1)
		throw std::system_error(
			std::error_code(errno, std::generic_category()),
			"Failed to epoll_create1");
}

NativeMainloop::~NativeMainloop()
{
	::close(m_pollfd);
}

void NativeMainloop::run(int timeout)
{
	this->m_isTimedOut = false;

	while (!this->m_isTimedOut) {
		this->dispatch(timeout);
	}

	DEBUG("NativeMainloop run stopped");
}

void NativeMainloop::addEventSource(
	int fd, Mainloop::Event event, Mainloop::Callback &&callback)
{
	std::lock_guard<std::mutex> l(this->m_mutex);

	if (this->m_callbacks.count(fd) != 0)
		ThrowExc(CSR_ERROR_SERVER, "event source on fd[" << fd << "] already added!");

	DEBUG("Add event[" << static_cast<uint32_t>(event)
		  << "] source on fd[" << fd << "]");

	epoll_event e;

	e.events = NativeMainloop::convertFlags(event);
	e.data.fd = fd;

	if (::epoll_ctl(m_pollfd, EPOLL_CTL_ADD, fd, &e) == -1)
		throw std::system_error(
			std::error_code(errno, std::generic_category()),
			"epoll_ctl failed to EPOLL_CTL_ADD.");

	this->m_callbacks[fd] = std::move(callback);
}

void NativeMainloop::removeEventSource(int fd)
{
	std::lock_guard<std::mutex> l(this->m_mutex);

	auto it = this->m_callbacks.find(fd);
	if (it == this->m_callbacks.end())
		ThrowExc(CSR_ERROR_SERVER, "event source on fd[" << fd << "] isn't added at all");

	DEBUG("Remove event source on fd[" << fd << "]");

	this->m_callbacks.erase(it);

	if (::epoll_ctl(m_pollfd, EPOLL_CTL_DEL, fd, nullptr) == -1) {
		if (errno == ENOENT)
			ThrowExc(CSR_ERROR_SERVER, "Tried to delete epoll item which wasn't added");
		else
			throw std::system_error(
				std::error_code(errno, std::generic_category()),
				"epoll_ctl failed to EPOLL_CTL_DEL.");
	}
}

void NativeMainloop::dispatch(int timeout)
{
	int nfds = -1;
	epoll_event event[MAX_EPOLL_EVENTS];

	DEBUG("NativeMainloop dispatched with timeout: " << timeout);

	do {
		nfds = ::epoll_wait(this->m_pollfd, event, MAX_EPOLL_EVENTS,
							((timeout < 0) ? -1 : (timeout * 1000)));
	} while ((nfds == -1) && (errno == EINTR));

	if (nfds < 0)
		throw std::system_error(
			std::error_code(errno, std::generic_category()),
			"epoll_wait failed!");

	if (nfds == 0) {
		DEBUG("NativeMainloop timed out!");
		if (this->m_callbacks.size() > this->m_domainSourceNum) {
			INFO("NativeMainloop timed out but there's running task on upper layer. "
				 "Re-dispatch.");
			this->m_isTimedOut = false;
		} else {
			INFO("NativeMainloop timed out! stop the loop!");
			this->m_isTimedOut = true;
		}

		return;
	}

	for (int i = 0; i < nfds; i++) {
		int fd = event[i].data.fd;

		auto it = this->m_callbacks.find(fd);
		if (it == this->m_callbacks.end())
			ThrowExc(CSR_ERROR_SERVER, "event in on fd[" << fd <<
					 "] but associated callback isn't exist!");

		auto events = convertFlags(event[i].events);
		if ((events & Mainloop::Event::CLOSE) != Mainloop::Event::NONE) {
			DEBUG("peer connection closed on fd[" << fd << "]");
			events &= ~Mainloop::Event::READ;
		}

		DEBUG("event[" << static_cast<uint32_t>(events)
			  << "] polled on fd[" << fd << "]");

		it->second(events);
	}
}

}
