/*
 *  Copyright (c) 2016 Samsung Electronics Co., Ltd All Rights Reserved
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License
 */
/*
 * @file        file-system.cpp
 * @author      Dongsun Lee (ds73.lee@samsung.com)
 * @version     1.0
 * @brief
 */
#include "service/file-system.h"

#include <regex>
#include <system_error>
#include <cstdio>
#include <cstring>
#include <cerrno>
#include <unistd.h>

#include "common/audit/logger.h"
#include "common/exception.h"
#include "service/app-deleter.h"
#include "service/app-dirs.h"
#include "service/fs-utils.h"
#include "service/dir-blacklist.h"

#include <pkgmgr-info.h>

namespace Csr {

File::Type File::Type::get(const std::string &user, const std::string &pkgid)
{
	pkgmgrinfo_pkginfo_h handle;

	int ret = -1;
	if (user.empty())
		ret = ::pkgmgrinfo_pkginfo_get_pkginfo(pkgid.c_str(), &handle);
	else
		ret = ::pkgmgrinfo_pkginfo_get_usr_pkginfo(pkgid.c_str(), getUid(user), &handle);

	Type type;

	if (ret != PMINFO_R_OK) {
		INFO("Extracted pkgid[" << pkgid << "] from filepath isn't pkg id. "
			 "It's not package.");
		return type;
	}

	type.m_attributes.set(Attribute::Package);

	bool isPreloaded = false;
	ret = ::pkgmgrinfo_pkginfo_is_preload(handle, &isPreloaded);

	if (ret != PMINFO_R_OK)
		ERROR("Failed to ::pkgmgrinfo_pkginfo_is_preload: " << ret);

	if (isPreloaded)
		type.m_attributes.set(Attribute::PreLoaded);

	::pkgmgrinfo_pkginfo_destroy_pkginfo(handle);

	return Type(type);
}

File::Type File::Type::get(const struct stat &s, time_t modifiedSince)
{
	Type type;

	if (S_ISREG(s.st_mode))
		type.m_attributes.set(Attribute::File);
	else
		type.m_attributes.set(Attribute::Directory);

	if (modifiedSince == -1 || s.st_ctime > modifiedSince)
		type.m_attributes.set(Attribute::Modified);

	return type;
}

std::string File::getPkgPath(const std::string &path)
{
	std::smatch matched;

	for (const auto &rege : AppDirs::get()) {
		if (!std::regex_search(path, matched, rege))
			continue;

		std::string pkgPath;
		std::string pkgUser;
		std::string pkgId;

		if (matched.size() == 3) {
			pkgPath = matched[1];
			pkgId = matched[2];
		} else if (matched.size() == 4) {
			pkgPath = matched[1];
			pkgUser = matched[2];
			pkgId = matched[3];
		} else {
			continue;
		}

		return Type::get(pkgUser, pkgId).isInApp() ? pkgPath : path;
	}

	return path;
}

File::File(const std::string &fpath, const FilePtr &parentdir, const Type &type,
		   std::unique_ptr<struct stat> &&statptr) :
	m_path(fpath), m_type(type), m_statptr(std::move(statptr))
{
	if (parentdir != nullptr) {
		if (parentdir->m_type.isPackage()) {
			this->m_appPkgPath = parentdir->getAppPkgPath();
			this->m_appPkgId = parentdir->getAppPkgId();
			this->m_appUser = parentdir->getAppUser();

			this->m_type.inheritParentType(parentdir->m_type);

			return;
		} else if (!this->isDir()) {
			this->m_type.unsetDirAttributes();
		}
	}

	std::smatch matched;

	for (const auto &rege : AppDirs::get()) {
		if (!std::regex_search(this->m_path, matched, rege))
			continue;

		if (matched.size() == 3) {
			this->m_appPkgPath = matched[1];
			this->m_appPkgId = matched[2];
			this->m_appUser.clear();
		} else if (matched.size() == 4) {
			this->m_appPkgPath = matched[1];
			this->m_appUser = matched[2];
			this->m_appPkgId = matched[3];
		} else {
			continue;
		}

		this->m_type |= Type::get(this->m_appUser, this->m_appPkgId);

		break;
	}
}

void File::remove() const
{
	if (this->isInApp()) {
		DEBUG("remove app: " << this->m_appPkgId);
		AppDeleter::remove(this->m_appPkgId, this->m_appUser);
	} else {
		DEBUG("remove file: " << this->m_path);
		if (::remove(this->m_path.c_str()) != 0)
			ThrowExc(CSR_ERROR_REMOVE_FAILED,
					 "Failed to remove file: " << this->m_path << " with errno: " << errno);
	}
}

FilePtr File::createIfModified(const std::string &fpath, const FilePtr &parentdir, time_t modifiedSince)
{
	return File::createInternal(fpath, parentdir, modifiedSince, true);
}

FilePtr File::create(const std::string &fpath, const FilePtr &parentdir, time_t modifiedSince)
{
	return File::createInternal(fpath, parentdir, modifiedSince, false);
}

FilePtr File::createInternal(const std::string &fpath, const FilePtr &parentdir,
							 time_t modifiedSince, bool isModifiedOnly)
{
	auto statptr = getStat(fpath);

	if (statptr == nullptr)
		ThrowExcWarn(CSR_ERROR_FILE_DO_NOT_EXIST, "file not exist or no permission: " <<
					 fpath);
	else if (!S_ISREG(statptr->st_mode) && !S_ISDIR(statptr->st_mode))
		ThrowExc(CSR_ERROR_FILE_SYSTEM, "file type is not reguler or dir: " << fpath);

	auto type = Type::get(*statptr, modifiedSince);

	if (isModifiedOnly && !type.isModified())
		return nullptr;
	else
		return FilePtr(new File(fpath, parentdir, type, std::move(statptr)));
}

std::map<std::string, std::mutex> FsVisitor::m_locks;
std::mutex FsVisitor::m_lockMapCleanMutex;

void FsVisitor::cleanLocks()
{
	std::lock_guard<std::mutex> l(FsVisitor::m_lockMapCleanMutex);
	FsVisitor::m_locks.clear();
}

FsVisitor::Dir::Dir(const std::string &dirname) : m_dir(nullptr), m_dirname(dirname)
{
	if (isInBlackList(this->m_dirname))
		throw std::invalid_argument("dirname is in blacklist.");

	DEBUG("try lock() on dir: " << this->m_dirname);
	// get lock ownership before open directory stream
	FsVisitor::m_locks[this->m_dirname].lock();
	DEBUG("get lock for dir: " << this->m_dirname);

	if ((this->m_dir = ::opendir(this->m_dirname.c_str())) == nullptr) {
		// release lock before throw exception because dtor won't be called.
		FsVisitor::m_locks[this->m_dirname].unlock();
		throw std::invalid_argument("cannot open dir");
	}
}

FsVisitor::Dir::~Dir()
{
	::closedir(this->m_dir);
	FsVisitor::m_locks[this->m_dirname].unlock();
	DEBUG("unlocked with closedir: " << this->m_dirname);
}

DIR *FsVisitor::Dir::get()
{
	return this->m_dir;
}

FsVisitor::DirPtr FsVisitor::openDir(const std::string &dir)
{
	try {
		return DirPtr(new FsVisitor::Dir(dir));
	} catch (...) {
		return DirPtr(nullptr);
	}
}

FsVisitorPtr FsVisitor::create(TargetHandler &&targetHandler,
							   const std::string &dirpath, bool isBasedOnName,
							   time_t modifiedSince)
{
	auto statptr = getStat(dirpath);
	if (statptr == nullptr)
		ThrowExcWarn(CSR_ERROR_FILE_DO_NOT_EXIST, "directory not exist or no "
					 "permission: " << dirpath);
	else if (!S_ISDIR(statptr->st_mode))
		ThrowExc(CSR_ERROR_FILE_SYSTEM, "file type is not directory: " << dirpath);
	else
		return FsVisitorPtr(new FsVisitor(std::move(targetHandler), dirpath,
										  isBasedOnName, modifiedSince));
}

FsVisitor::FsVisitor(TargetHandler &&targetHandler, const std::string &dirpath,
					 bool isBasedOnName, time_t modifiedSince) :
	m_targetHandler(std::move(targetHandler)), m_path(dirpath),
	m_since(modifiedSince), m_isDone(true), m_isBasedOnName(isBasedOnName)
{
}

FsVisitor::~FsVisitor()
{
}

void FsVisitor::run(const DirPtr &dirptr, const FilePtr &currentdir)
{
	while (true) {
		errno = 0;
		auto result = ::readdir(dirptr->get());
		if (result == nullptr) {
			if (errno != 0)
				WARN("readdir error on dir: " << currentdir->getPath() <<
					 " with errno: " << errno << ". Silently ignore this error &"
					 " dir stream to reduce side-effect of traversing all the other"
					 " file systems.");
			else
				DEBUG("end of stream: " << currentdir->getPath());

			break;
		}

		const auto &name = result->d_name;
		auto name_size = ::strlen(name);

		if (name_size == 0)
			continue;

		const auto &parent_dirpath = currentdir->getPath();
		auto fullpath = (parent_dirpath.back() == '/') ?
				(parent_dirpath + name) : (parent_dirpath + "/" + name);

		DEBUG("start to traverse file: " << fullpath);

		if (result->d_type == DT_LNK) {
			if (!currentdir->isInApp())
				continue;

			auto stat = getStat(fullpath);
			if (stat == nullptr) {
				WARN("Skip the file(" << fullpath << ") whose stat can't be retrieved");
				continue;
			}

			if (S_ISREG(stat->st_mode))
				this->visitFile(fullpath, currentdir);
			else if (S_ISDIR(stat->st_mode))
				this->visitDir(fullpath, currentdir);

		} else if (result->d_type == DT_DIR) {
			if ((name_size == 1 && name[0] == '.') ||
				(name_size == 2 && name[0] == '.' && name[1] == '.'))
				continue;

			this->visitDir(fullpath, currentdir);
		} else if (result->d_type == DT_REG) {
			this->visitFile(fullpath, currentdir);
		}
	}
}

void FsVisitor::visitDir(const std::string &fullpath, const FilePtr &currentdir)
{
	FilePtr ncurrentdir;

	try {
		ncurrentdir = File::create(fullpath, currentdir);
	} catch (const Exception &e) {
		if (e.error() == CSR_ERROR_FILE_DO_NOT_EXIST) {
			WARN("Perm denied to create file on pkg path: " << fullpath);
			return;
		} else {
			throw;
		}
	}

	if (this->m_isBasedOnName && ncurrentdir->isInApp()) {
		this->m_targetHandler(ncurrentdir);
	} else {
		if (auto ndirptr = openDir(fullpath)) {
			DEBUG("recurse dir: " << fullpath);
			this->run(ndirptr, ncurrentdir);
		} else {
			WARN("Failed to open dir: " << fullpath << " with errno: " << errno);
		}
	}
}

void FsVisitor::visitFile(const std::string &fullpath, const FilePtr &currentdir)
{
	try {
		if (auto fileptr = File::createIfModified(fullpath, currentdir, this->m_since))
			this->m_targetHandler(fileptr);
	} catch (const Exception &e) {
		if (e.error() == CSR_ERROR_FILE_DO_NOT_EXIST)
			WARN("file not exist: " << fullpath << " msg: " << e.what());
		else if (e.error() == CSR_ERROR_FILE_SYSTEM)
			WARN("file type is not regular...? can it be happened?"
				 " :" << fullpath << " msg: " << e.what());
		else
			throw;
	}
}

void FsVisitor::run()
{
	auto currentdir = File::create(this->m_path, nullptr);

	INFO("Visiting files start from dir: " << this->m_path);

	if (this->m_isBasedOnName && currentdir->isInApp()) {
		this->m_targetHandler(currentdir);
		return;
	}

	auto dirptr = openDir(this->m_path);

	if (dirptr == nullptr) {
		int err = errno;
		WARN("Failed to open dir: " << this->m_path << " with errno: " << err <<
			 ". let's retry.");
		dirptr = openDir(this->m_path);
		if (dirptr == nullptr) {
			err = errno;
			WARN("Failed to open dir again: " << this->m_path << " with errno: " << err <<
				 ". let's skip.");
			return;
		}
	}

	this->run(dirptr, currentdir);
}

} // namespace Csr
