/*
 *  Copyright (c) 2016 Samsung Electronics Co., Ltd All Rights Reserved
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License
 */
/*
 * @file        file-system.h
 * @author      Dongsun Lee (ds73.lee@samsung.com)
 * @version     1.0
 * @brief
 */
#pragma once

#include <string>
#include <memory>
#include <queue>
#include <functional>
#include <mutex>
#include <map>
#include <bitset>

#include <cstddef>
#include <ctime>
#include <dirent.h>
#include <sys/stat.h>

namespace Csr {

class File;
using FilePtr = std::unique_ptr<File>;

class File {
public:
	File() = delete;

	inline bool isModifiedSince(time_t since) const noexcept
	{
		return this->m_statptr->st_ctime > since;
	}

	inline const std::string &getName() const noexcept
	{
		return (this->isInApp()) ? this->m_appPkgPath : this->m_path;
	}

	inline const std::string &getPath() const noexcept
	{
		return this->m_path;
	}

	inline const std::string &getAppPkgId() const noexcept
	{
		return this->m_appPkgId;
	}

	inline const std::string &getAppUser() const noexcept
	{
		return this->m_appUser;
	}

	inline const std::string &getAppPkgPath() const noexcept
	{
		return this->m_appPkgPath;
	}

	inline bool isInApp() const noexcept
	{
		return this->m_type.isInApp();
	}

	inline bool isDir() const noexcept
	{
		return this->m_type.isDir();
	}

	void remove() const;

	// throws FileNotExist and FileSystemError
	static FilePtr create(const std::string &fpath, const FilePtr &parentdir,
						  time_t modifiedSince = -1);
	static FilePtr createIfModified(const std::string &fpath, const FilePtr &parentdir,
									time_t modifiedSince = -1);

	static std::string getPkgPath(const std::string &path);

private:
	class Type {
	public:
		inline Type &operator|=(const Type &other)
		{
			this->m_attributes |= other.m_attributes;
			return *this;
		}

		inline bool isInApp() const noexcept
		{
			return Type::isInApp(this->m_attributes);
		}

		inline bool isPackage() const noexcept
		{
			return Type::isPackage(this->m_attributes);
		}

		inline bool isPreloaded() const noexcept
		{
			return Type::isPreloaded(this->m_attributes);
		}

		inline bool isModified() const noexcept
		{
			return Type::isModified(this->m_attributes);
		}

		inline bool isDir() const noexcept
		{
			return Type::isDir(this->m_attributes);
		}

		inline void unsetDirAttributes() noexcept
		{
			this->m_attributes.reset(Attribute::Package);
			this->m_attributes.reset(Attribute::PreLoaded);
		}

		inline void inheritParentType(const Type &parent) noexcept
		{
			if (parent.isPackage())
				this->m_attributes.set(Attribute::Package);
			if (parent.isPreloaded())
				this->m_attributes.set(Attribute::PreLoaded);
		}

		static Type get(const std::string &user, const std::string &pkgid);
		static Type get(const struct stat &s, time_t modifiedSince);

	private:
		enum Attribute {
			Modified  = 0,
			Package   = 1,
			PreLoaded = 2,
			File      = 3,
			Directory = 4,
			ENUM_COUNT = 5
		};

		using Attributes = std::bitset<static_cast<std::underlying_type<Attribute>::type>(Attribute::ENUM_COUNT)>;

		static inline bool isInApp(const Attributes &attributes) noexcept
		{
			return isPackage(attributes) && !isPreloaded(attributes);
		}

		static inline bool isPackage(const Attributes &attributes) noexcept
		{
			return attributes[Attribute::Package];
		}

		static inline bool isPreloaded(const Attributes &attributes) noexcept
		{
			return attributes[Attribute::PreLoaded];
		}

		static inline bool isModified(const Attributes &attributes) noexcept
		{
			return attributes[Attribute::Modified];
		}

		static inline bool isDir(const Attributes &attributes) noexcept
		{
			return attributes[Attribute::Directory];
		}

		Type() {}

		Attributes m_attributes;
	};

	static FilePtr createInternal(const std::string &fpath, const FilePtr &parentdir,
								  time_t modifiedSince, bool isModifiedOnly);

	explicit File(const std::string &fpath, const FilePtr &parentdir,
				  const Type &type, std::unique_ptr<struct stat> &&statptr);

	std::string m_path;
	Type m_type;
	std::unique_ptr<struct stat> m_statptr;
	std::string m_appPkgId;    // meaningful only if inApp == true
	std::string m_appUser;     // meaningful only if inApp == true
	std::string m_appPkgPath;  // meaningful only if inApp == true
};

// visitor traverses file which is modified since the time of 'modifiedSince'
// if modifiedSince is -1, traverse regardless of time
class FsVisitor;
using FsVisitorPtr = std::unique_ptr<FsVisitor>;

class FsVisitor {
public:
	using TargetHandler = std::function<void(FilePtr &)>;

	~FsVisitor();

	void run();

	// throws FileNotExist and FileSystemError
	static FsVisitorPtr create(TargetHandler &&targetHandler,
							   const std::string &dirpath, bool isBasedOnName,
							   time_t modifiedSince = -1);

	// clean up per directory lock map.
	// Warning: should be called only no directory is traversed at the time.
	// this is essential because mutex exists per directory and it can be
	// stacked a lot because it won't be freed unless process is terminated.
	static void cleanLocks();

private:
	class Dir {
	public:
		Dir(const std::string &dirname);
		~Dir();
		DIR *get();

	private:
		DIR *m_dir;
		std::string m_dirname;
	};

	using DirPtr = std::unique_ptr<Dir>;

	void run(const DirPtr &dirptr, const FilePtr &currentdir);
	void visitDir(const std::string &fullpath, const FilePtr &currentdir);
	void visitFile(const std::string &fullpath, const FilePtr &currentdir);

	static DirPtr openDir(const std::string &);

	FsVisitor(TargetHandler &&targetHandler, const std::string &dirpath,
			  bool isBasedOnName, time_t modifiedSince = -1);

	static std::map<std::string, std::mutex> m_locks;
	static std::mutex m_lockMapCleanMutex;

	TargetHandler m_targetHandler;
	std::string m_path;
	time_t m_since;
	bool m_isDone;
	bool m_isBasedOnName;
};

} // namespace Csr
