/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd All Rights Reserved
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

#include "ecore-mainloop.h"

#include <utility>
#include <exception>

#include "common/audit/logger.h"

namespace Csr {
namespace Ui {

Ecore_Fd_Handler_Flags EcoreMainloop::convertFlags(Mainloop::Event events)
{
	uint32_t flags = 0;

	if ((events & Mainloop::Event::READ) != Mainloop::Event::NONE)
		flags |= ECORE_FD_READ;
	if ((events & Mainloop::Event::WRITE) != Mainloop::Event::NONE)
		flags |= ECORE_FD_WRITE;

	return static_cast<Ecore_Fd_Handler_Flags>(flags);
}

Mainloop::Event EcoreMainloop::convertFlags(Ecore_Fd_Handler *handler)
{
	Mainloop::Event flags = Mainloop::Event::NONE;

	if (ecore_main_fd_handler_active_get(handler, ECORE_FD_READ))
		flags |= Mainloop::Event::READ;
	if (ecore_main_fd_handler_active_get(handler, ECORE_FD_WRITE))
		flags |= Mainloop::Event::WRITE;
	if (ecore_main_fd_handler_active_get(handler, ECORE_FD_ERROR))
		flags |= Mainloop::Event::CLOSE;

	return flags;
}

EcoreMainloop::EcoreMainloop() : m_isExitCalled(false), m_timer(nullptr) {}

EcoreMainloop::~EcoreMainloop()
{
	if (this->m_timer)
		ecore_timer_del(this->m_timer);

	if (!this->m_isExitCalled)
		elm_exit();
}

void EcoreMainloop::run(int timeout)
{
	this->dispatch(timeout);
}

void EcoreMainloop::dispatch(int timeout)
{
	if (timeout >= 0) {
		this->m_timer = ecore_timer_add(
			static_cast<double>(timeout), &EcoreMainloop::ecoreTimeoutCb, this);

		if (this->m_timer == nullptr) {
			ERROR("Failed to ecore_timer_add()...");
		}
	}

	elm_run();
}

void EcoreMainloop::addEventSource(
	int fd, Mainloop::Event events, Mainloop::Callback &&callback)
{
	auto handler = ecore_main_fd_handler_add(
		fd, convertFlags(events),
		&EcoreMainloop::ecoreFdCallback, this, nullptr, nullptr);

	if (handler == nullptr) {
		ERROR("failed to handle fd(" << fd << ") by ecore_main_fd_handler_add()");
		return;
	}

	std::unique_ptr<EventSource> source(new EventSource);
	source->callback = std::move(callback);
	source->handler = handler;

	this->m_sources[fd] = std::move(source);
}

void EcoreMainloop::removeEventSource(int fd)
{
	auto it = this->m_sources.find(fd);

	if (it == this->m_sources.end()) {
		ERROR("fd(" << fd << ") associated source is not found");
		return;
	}

	ecore_main_fd_handler_del(it->second->handler);

	this->m_sources.erase(it);
}

Eina_Bool EcoreMainloop::ecoreFdCallback(void *data, Ecore_Fd_Handler *handler)
{
	auto mainloop = static_cast<EcoreMainloop *>(data);

	if (mainloop == nullptr)
		throw std::invalid_argument("userdata for ecore fd callback is invalid");

	int fd = ecore_main_fd_handler_fd_get(handler);

	auto it = mainloop->m_sources.find(fd);
	if (it == mainloop->m_sources.end() || it->second == nullptr) {
		ERROR("no associated source found with fd: " << fd);
		return ECORE_CALLBACK_CANCEL;
	}

	it->second->callback(convertFlags(handler));

	return ECORE_CALLBACK_RENEW;
}

Eina_Bool EcoreMainloop::ecoreTimeoutCb(void *data)
{
	auto mainloop = static_cast<EcoreMainloop *>(data);

	if (mainloop == nullptr)
		throw std::invalid_argument("userdata for ecore timeout callback is invalid");

	if (mainloop->m_sources.size() == mainloop->m_domainSourceNum) {
		INFO("There's no alive connection. Only listening socket opened. "
			 "Let's exit the ecore main loop!");

		elm_exit();

		mainloop->m_timer = nullptr;
		mainloop->m_isExitCalled = true;

		return ECORE_CALLBACK_CANCEL;
	} else {
		INFO("Time out expired but there's alive connection "
			 "so go ahead to next tick!");
		return ECORE_CALLBACK_RENEW;
	}
}

} // namespace Ui
} // namespace Csr
