/*
 *  Copyright (c) 2016 Samsung Electronics Co., Ltd All Rights Reserved
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License
 */
/*
 * @file        logic.cpp
 * @author      Kyungwook Tak (k.tak@samsung.com)
 * @author      Sangwan Kwon (sangwan.kwon@samsung.com)
 * @version     1.0
 * @brief       Logic for popup service
 */
#include "logic.h"

#include <functional>
#include <unordered_map>
#include <Elementary.h>
#include <limits.h>
#include <vector>

#include "common/binary-queue.h"
#include "common/audit/logger.h"
#include "common/exception.h"
#include "ui/common.h"
#include "package-info.h"
#include "popup.h"
#include "popup-string.h"

#include <csr-content-screening-types.h>
#include <csr-web-protection-types.h>

#ifndef PATH_MAX
#define PATH_MAX 4096
#endif


namespace Csr {
namespace Ui {

namespace {
void split(const std::string &s, std::string &fileName, std::string &extraPath)
{
	std::string delimiter = "/";
	size_t idx = s.rfind(delimiter);

	fileName = s.substr(idx + 1);
	extraPath = s.substr(0, idx);
}

std::string formatToString(const char *form, const std::string &arg)
{
	std::vector<char> buf(PATH_MAX, '\0');
	snprintf(buf.data(), buf.size(), form, arg.c_str());
	return std::string((buf.data()));
}
} // namespace anonymous

void Logic::csPromptData(const std::string &message, const CsDetected &d)
{
	std::string risk(d.severity ==
		CSR_CS_SEVERITY_LOW ? LABEL_RISK_LEVEL_LOW : LABEL_RISK_LEVEL_MEDIUM);

	m_popup.reset();
	m_popup.setMessage(message);
	m_popup.setTitle(CS_TITLE);
	m_popup.setHeader(CS_PROMPT_DATA_HEADER);
	m_popup.setBody(FORMAT(
		"- " << LABEL_RISK << risk << " (" << d.malwareName << ")"));
	m_popup.setFooter(CS_PROMPT_DATA_FOOTER);
	m_popup.setDetailedUrl(d.detailedUrl);

	m_popup.setButtons({
		Popup::Button(BTN_CANCEL, CSR_CS_USER_RESPONSE_PROCESSING_DISALLOWED),
		Popup::Button(BTN_OPEN, CSR_CS_USER_RESPONSE_PROCESSING_ALLOWED)});
}

void Logic::csPromptFile(const std::string &message, const CsDetected &d)
{
	std::string risk(d.severity ==
		CSR_CS_SEVERITY_LOW ? LABEL_RISK_LEVEL_LOW : LABEL_RISK_LEVEL_MEDIUM);
	std::string fileName, extraPath;
	split(d.targetName, fileName, extraPath);

	m_popup.reset();
	m_popup.setMessage(message);
	m_popup.setTitle(CS_TITLE);
	m_popup.setHeader(CS_PROMPT_FILE_HEADER);
	m_popup.setBody(FORMAT(
		"- " << formatToString(LABEL_FILE_NAME, fileName) << "<br>" <<
		"- " << formatToString(LABEL_FILE_PATH, extraPath) << "<br>" <<
		"- " << LABEL_RISK << risk << " (" << d.malwareName << ")"));
	std::string prefix(CS_NOTIFY_FILE_FOOTER);
	m_popup.setFooter(FORMAT(prefix << "<br>" << CS_PROMPT_FILE_FOOTER));
	m_popup.setDetailedUrl(d.detailedUrl);

	m_popup.setButtons({
		Popup::Button(BTN_CANCEL, CSR_CS_USER_RESPONSE_PROCESSING_DISALLOWED),
		Popup::Button(BTN_OPEN, CSR_CS_USER_RESPONSE_PROCESSING_ALLOWED),
		Popup::Button(BTN_DELETE, CSR_CS_USER_RESPONSE_REMOVE)});
}

void Logic::csPromptApp(const std::string &message, const CsDetected &d)
{
	std::string risk(d.severity ==
		CSR_CS_SEVERITY_LOW ? LABEL_RISK_LEVEL_LOW : LABEL_RISK_LEVEL_MEDIUM);
	PackageInfo info(d.pkgId);

	m_popup.reset();
	m_popup.setMessage(message);
	m_popup.setTitle(CS_TITLE);
	m_popup.setHeader(CS_PROMPT_APP_HEADER);
	m_popup.setBody(FORMAT(
		LABEL_APP_NAME << info.getLabel() << "<br>" <<
		LABEL_VERSION << info.getVersion() << "<br>" <<
		LABEL_RISK << risk << " (" << d.malwareName << ")"));
	m_popup.setIcon(info.getIconPath());
	std::string prefix = formatToString(CS_NOTIFY_APP_FOOTER, BTN_UNINSTALL);
	m_popup.setFooter(FORMAT(prefix << "<br>" << CS_PROMPT_APP_FOOTER));
	m_popup.setDetailedUrl(d.detailedUrl);

	m_popup.setButtons({
		Popup::Button(BTN_CANCEL, CSR_CS_USER_RESPONSE_PROCESSING_DISALLOWED),
		Popup::Button(BTN_OPEN, CSR_CS_USER_RESPONSE_PROCESSING_ALLOWED),
		Popup::Button(BTN_UNINSTALL, CSR_CS_USER_RESPONSE_REMOVE)});
}

void Logic::csNotifyData(const std::string &message, const CsDetected &d)
{
	m_popup.reset();
	m_popup.setMessage(message);
	m_popup.setTitle(CS_TITLE);
	m_popup.setHeader(CS_NOTIFY_DATA_HEADER);
	m_popup.setBody(FORMAT(
		"- " << LABEL_RISK << LABEL_RISK_LEVEL_HIGH <<
		" (" << d.malwareName << ")"));
	m_popup.setFooter(CS_NOTIFY_DATA_FOOTER);
	m_popup.setDetailedUrl(d.detailedUrl);

	m_popup.setButtons({
		Popup::Button(BTN_OK, CSR_CS_USER_RESPONSE_PROCESSING_DISALLOWED)});
}

void Logic::csNotifyFile(const std::string &message, const CsDetected &d)
{
	std::string fileName, extraPath;
	split(d.targetName, fileName, extraPath);

	m_popup.reset();
	m_popup.setMessage(message);
	m_popup.setTitle(CS_TITLE);
	m_popup.setHeader(CS_NOTIFY_FILE_HEADER);
	m_popup.setBody(FORMAT(
		"- " << formatToString(LABEL_FILE_NAME, fileName) << "<br>" <<
		"- " << formatToString(LABEL_FILE_PATH, extraPath) << "<br>" <<
		"- " << LABEL_RISK << LABEL_RISK_LEVEL_HIGH <<
		" (" << d.malwareName << ")"));
	m_popup.setFooter(CS_NOTIFY_FILE_FOOTER);
	m_popup.setDetailedUrl(d.detailedUrl);

	m_popup.setButtons({
		Popup::Button(BTN_CANCEL, CSR_CS_USER_RESPONSE_PROCESSING_DISALLOWED),
		Popup::Button(BTN_DELETE, CSR_CS_USER_RESPONSE_REMOVE)});
}

void Logic::csNotifyApp(const std::string &message, const CsDetected &d)
{
	PackageInfo info(d.pkgId);

	m_popup.reset();
	m_popup.setMessage(message);
	m_popup.setTitle(CS_TITLE);
	m_popup.setHeader(CS_NOTIFY_APP_HEADER);
	m_popup.setIcon(info.getIconPath());
	m_popup.setBody(FORMAT(
		LABEL_APP_NAME << info.getLabel() << "<br>" <<
		LABEL_VERSION << info.getVersion() << "<br>" <<
		LABEL_RISK << LABEL_RISK_LEVEL_HIGH << " (" << d.malwareName << ")"));
	m_popup.setFooter(formatToString(CS_NOTIFY_APP_FOOTER, BTN_UNINSTALL));
	m_popup.setDetailedUrl(d.detailedUrl);

	m_popup.setButtons({
		Popup::Button(BTN_CANCEL, CSR_CS_USER_RESPONSE_PROCESSING_DISALLOWED),
		Popup::Button(BTN_UNINSTALL, CSR_CS_USER_RESPONSE_REMOVE)});
}

void Logic::wpPrompt(const std::string &message, const UrlItem &item)
{
	std::string risk(item.risk ==
		CSR_WP_RISK_LOW ? LABEL_RISK_LEVEL_LOW : LABEL_RISK_LEVEL_MEDIUM);

	m_popup.reset();
	m_popup.setMessage(message);
	m_popup.setTitle(WP_TITLE);
	m_popup.setHeader(WP_PROMPT_HEADER);
	m_popup.setBody(FORMAT(
		"- " << formatToString(LABEL_URL, item.url) << "<br>" <<
		"- " << LABEL_RISK << risk));
	m_popup.setFooter(WP_PROMPT_FOOTER);
	m_popup.setDetailedUrl(item.url);

	m_popup.setButtons({
		Popup::Button(BTN_BLOCK, CSR_WP_USER_RESPONSE_PROCESSING_DISALLOWED),
		Popup::Button(BTN_VIEW, CSR_WP_USER_RESPONSE_PROCESSING_ALLOWED)});
}

void Logic::wpNotify(const std::string &message, const UrlItem &item)
{
	m_popup.reset();
	m_popup.setMessage(message);
	m_popup.setTitle(WP_TITLE);
	m_popup.setHeader(WP_NOTIFY_HEADER);
	m_popup.setBody(FORMAT(
		"- " << formatToString(LABEL_URL, item.url) << "<br>" <<
		"- " << LABEL_RISK << LABEL_RISK_LEVEL_HIGH));
	m_popup.setFooter(WP_NOTIFY_FOOTER);
	m_popup.setDetailedUrl(item.url);

	m_popup.setButtons({
		Popup::Button(BTN_OK, CSR_WP_USER_RESPONSE_PROCESSING_DISALLOWED)});
}
} // namespace Ui
} // namespace Csr
