/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd All Rights Reserved
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */

#include "popup-service.h"

#include "common/binary-queue.h"
#include "common/audit/logger.h"
#include "common/types.h"
#include "common/exception.h"

#include "ecore-mainloop.h"

namespace Csr {
namespace Ui {

namespace {

bool isCsCommand(const CommandId &cid)
{
	switch (cid) {
	case CommandId::CS_PROMPT_DATA:
	case CommandId::CS_PROMPT_APP:
	case CommandId::CS_PROMPT_FILE:
	case CommandId::CS_NOTIFY_DATA:
	case CommandId::CS_NOTIFY_APP:
	case CommandId::CS_NOTIFY_FILE:
		return true;

	case CommandId::WP_PROMPT:
	case CommandId::WP_NOTIFY:
		return false;

	default:
		ThrowExc(CSR_ERROR_SERVER, "Protocol error. unknown popup-service command id.");
	}
}

} // namespace anonymous

PopupService::PopupService() : Csr::Service(new EcoreMainloop())
{
	this->add(SockId::POPUP);
}

void PopupService::onMessageProcess(const ConnShPtr &conn)
{
	auto in = conn->receive();

	// some fd handler from mainloop makes empty read event when client
	// close connection. returning false from process() means connection
	// is closed
	if (in.empty())
		throw std::invalid_argument("empty read event!");

	BinaryQueue q;
	q.push(std::move(in));

	int intCid;
	q.Deserialize(intCid);

	INFO("Request dispatch on popup service. CommandId: " << static_cast<int>(intCid));

	if (isCsCommand(static_cast<Ui::CommandId>(intCid))) {
		std::string message;
		CsDetected d;
		q.Deserialize(message, d);

		switch (static_cast<Ui::CommandId>(intCid)) {
		case CommandId::CS_PROMPT_DATA:
			m_logic.csPromptData(message, d);
			break;

		case CommandId::CS_PROMPT_APP:
			m_logic.csPromptApp(message, d);
			break;

		case CommandId::CS_PROMPT_FILE:
			m_logic.csPromptFile(message, d);
			break;

		case CommandId::CS_NOTIFY_DATA:
			m_logic.csNotifyData(message, d);
			break;

		case CommandId::CS_NOTIFY_APP:
			m_logic.csNotifyApp(message, d);
			break;

		case CommandId::CS_NOTIFY_FILE:
			m_logic.csNotifyFile(message, d);
			break;

		default:
			ThrowExc(CSR_ERROR_SERVER, "protocol error. invalid ui command id.");
		}
	} else {
		std::string message;
		UrlItem item;
		q.Deserialize(message, item);

		switch (static_cast<Ui::CommandId>(intCid)) {
		case CommandId::WP_PROMPT:
			m_logic.wpPrompt(message, item);
			break;

		case CommandId::WP_NOTIFY:
			m_logic.wpNotify(message, item);
			break;

		default:
			ThrowExc(CSR_ERROR_SERVER, "protocol error. invalid ui command id.");
		}
	}

	m_logic.setSender([conn](const RawBuffer &out) { conn->send(out); });
}

} // namespace Ui
} // namespace Csr
