/*
 *  Copyright (c) 2016 Samsung Electronics Co., Ltd All Rights Reserved
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License
 */
/*
 * @file        popup.cpp
 * @author      Kyungwook Tak (k.tak@samsung.com)
 * @author      Sangwan Kwon (sangwan.kwon@samsung.com)
 * @version     1.0
 * @brief
 */
#include "popup.h"
#include "popup-string.h"

#include <package-info.h>
#include <app_control.h>
#include <efl_extension.h>

namespace Csr {
namespace Ui {

namespace {
	struct AppControl {
		AppControl() { app_control_create(&handle); }
		~AppControl() { app_control_destroy(handle); }

		app_control_h handle = nullptr;
	};

	const std::string DEFAULT_URL("https://developer.tizen.org/");
	const std::string HOME_KEY("XF86Home");

}

void Popup::setSender(Sender &&sender)
{
	m_sender = std::move(sender);
}

Popup::Popup()
{
	ecore_event_handler_add(ECORE_EVENT_KEY_DOWN, keyDownCb, this);
}

void Popup::reset()
{
	clear();

	// Set win properties.
	m_win = elm_win_add(nullptr, "CSR popup", ELM_WIN_NOTIFICATION);
	elm_win_indicator_opacity_set(m_win, ELM_WIN_INDICATOR_TRANSLUCENT);
	elm_win_borderless_set(m_win, EINA_TRUE);
	elm_win_alpha_set(m_win, EINA_TRUE);
	elm_win_screen_size_get(m_win, NULL, NULL, &m_winW, &m_winH);
	evas_object_size_hint_max_set(m_win, m_winW, m_winH);
	evas_object_size_hint_min_set(m_win, m_winW, m_winH / 4);
	setRotationToWin(m_win);

	eext_win_keygrab_set(m_win, HOME_KEY.c_str());

	// Set popup properties.
	m_popup = elm_popup_add(m_win);
	setDefaultProperties(m_popup);

	// Wrap objects with box.
	m_box = elm_box_add(m_popup);
	setDefaultProperties(m_box);
	elm_box_padding_set(m_box, 0, 20);
	evas_object_show(m_box);

	m_header = elm_label_add(m_box);
	setDefaultProperties(m_header);
	elm_box_pack_end(m_box, m_header);
	evas_object_show(m_header);

	// Subbox is for icon.
	m_subBox = elm_box_add(m_box);
	setDefaultProperties(m_subBox);
	elm_box_horizontal_set(m_subBox, EINA_TRUE);
	elm_box_padding_set(m_subBox, 20, 0);
	// If icon is not set, it doesn't appear.
	m_icon = elm_icon_add(m_subBox);
	elm_image_resizable_set(m_icon, EINA_FALSE, EINA_FALSE);
	elm_box_pack_end(m_subBox, m_icon);
	evas_object_show(m_icon);

	m_body = elm_label_add(m_subBox);
	setDefaultProperties(m_body);
	elm_box_pack_end(m_subBox, m_body);
	evas_object_show(m_body);

	elm_box_pack_end(m_box, m_subBox);
	evas_object_show(m_subBox);

	// This label is for linking to webview.
	m_hypertext = elm_label_add(m_box);
	setDefaultProperties(m_hypertext);
	setText(m_hypertext, FORMAT("<a href=><color=#0000FFFF>"
		<< LABEL_MORE_INFO << "</color></a>"));
	elm_box_pack_end(m_box, m_hypertext);
	evas_object_show(m_hypertext);

	m_footer = elm_label_add(m_box);
	setDefaultProperties(m_footer);
	elm_box_pack_end(m_box, m_footer);
	evas_object_show(m_footer);

	m_btnBox = elm_box_add(m_popup);
	setDefaultProperties(m_btnBox);
	elm_box_horizontal_set(m_btnBox, EINA_TRUE);
	elm_box_padding_set(m_btnBox, 0, 0);

	elm_box_pack_end(m_box, m_btnBox);
	evas_object_show(m_btnBox);

	elm_object_content_set(m_popup, m_box);

	evas_object_show(m_popup);
	evas_object_show(m_win);
}

Popup::~Popup()
{
	if (m_win != nullptr)
		evas_object_del(m_win);
}

void Popup::setButtons(const std::vector<Popup::Button> &buttons)
{
	for (auto &button : buttons) {
		Evas_Object *obj = elm_button_add(m_btnBox);

		elm_object_style_set(obj, "bottom");
		setDefaultProperties(obj);

		setText(obj, button.text);
		m_buttonTypeMap[obj] = button.type;
		evas_object_smart_callback_add(obj, "clicked", btnClickedCb, this);

		elm_box_pack_end(m_btnBox, obj);
		evas_object_show(obj);
	}
}

void Popup::setHeader(const std::string &header) noexcept
{
	setText(m_header, header);
}

void Popup::setBody(const std::string &body) noexcept
{
	setText(m_body, body);
}

void Popup::setFooter(const std::string &footer) noexcept
{
	setText(m_footer, footer);
}

void Popup::setMessage(const std::string &msg) noexcept
{
	m_message = msg;
}

void Popup::setIcon(const std::string &path) noexcept
{
	if (path.compare(PackageInfo::UNKNOWN) == 0)
		elm_image_file_set(m_icon, DEFAULT_ICON_PATH, NULL);
	else
		elm_image_file_set(m_icon, path.c_str(), NULL);
}

int Popup::response = -1;

RawBuffer Popup::getResult(void)
{
	return BinaryQueue::Serialize(response).pop();
}

void Popup::setTitle(const std::string &title) noexcept
{
	elm_object_part_text_set(m_popup, "title,text", title.c_str());
}

void Popup::setDefaultProperties(Evas_Object *obj) noexcept
{
	// Set width as maximum, height as minimum.
	evas_object_size_hint_weight_set(obj, EVAS_HINT_EXPAND, 0);
	// Set width as fill parent, height as center.
	evas_object_size_hint_align_set(obj, EVAS_HINT_FILL, 0.5);
}

void Popup::setRotationToWin(Evas_Object *win) noexcept
{
	if (!elm_win_wm_rotation_supported_get(win)) {
		DEBUG("Window manager doesn't support rotation.");
		return;
	}

	int rots[4] = { 0, 90, 180, 270 };
	elm_win_wm_rotation_available_rotations_set(win,
		reinterpret_cast<const int *>(&rots), 4);
	evas_object_smart_callback_add(win, "rotation,changed",
		rotationChangedCb, m_popup);
}

void Popup::setText(Evas_Object *obj, const std::string &text) noexcept
{
	// Enable text line-break automatically.
	elm_label_line_wrap_set(obj, ELM_WRAP_WORD);
	elm_object_text_set(obj, text.c_str());
}

void Popup::setDetailedUrl(const std::string &url)
{
	if (url.empty())
		m_hypertextUrl = DEFAULT_URL;
	else
		m_hypertextUrl = url;

	evas_object_smart_callback_add(
		m_hypertext, "anchor,clicked", hypertextClickedCb, this);
}

void Popup::rotationChangedCb(void *data, Evas_Object *, void *)
{
	DEBUG("Window rotation change event caught.");
	auto popup = reinterpret_cast<Evas_Object *>(data);
	auto win = elm_object_top_widget_get(popup);
	auto pos = elm_win_rotation_get(win);
	Evas_Coord w, h;
	elm_win_screen_size_get(win, NULL, NULL, &w, &h);

	switch (pos) {
	case 0:
	case 180:
		evas_object_move(popup, (w / 2), h);
		return;
	case 90:
	case 270:
		evas_object_move(popup, (h / 2), w);
		return;
	default:
		DEBUG("Invalid pos value : " << pos);
		return;
	}
}

void Popup::hypertextClickedCb(void *data, Evas_Object *, void *)
{
	DEBUG("Launch browser for detailed url.");
	auto popup = reinterpret_cast<Popup *>(data);
	if (popup == nullptr)
		throw std::invalid_argument("Popup instance is null in static callback");

	auto &url = popup->m_hypertextUrl;

	std::unique_ptr<AppControl> ac(new AppControl);

	auto ret = app_control_set_operation(ac->handle, APP_CONTROL_OPERATION_VIEW);
	if (ret != APP_CONTROL_ERROR_NONE) {
		WARN("Cannot set app_control operation.");
		return;
	}

	ret = app_control_set_uri(ac->handle, url.c_str());
	if (ret != APP_CONTROL_ERROR_NONE) {
		WARN("Cannot set url to app_control handle.");
		return;
	}

	ret = app_control_send_launch_request(ac->handle, NULL, NULL);
	if (ret != APP_CONTROL_ERROR_NONE) {
		WARN("Cannot launch browser.");
		return;
	}
}

void Popup::btnClickedCb(void *data, Evas_Object *obj, void *)
{
	auto popup = reinterpret_cast<Popup *>(data);
	if (popup == nullptr)
		throw std::invalid_argument("Popup instance is null in static btnClickedCb");

	response = popup->m_buttonTypeMap.at(obj);
	popup->terminateWindow();
}

Eina_Bool Popup::keyDownCb(void *data, int , void *ev)
{
	DEBUG("Key down event caught.");
	auto event = reinterpret_cast<Ecore_Event_Key *>(ev);

	if (event->key == HOME_KEY) {
		auto popup = reinterpret_cast<Popup *>(data);
		response = -1;
		popup->terminateWindow();
	}

	// Let the event continue to other callbacks.
	return ECORE_CALLBACK_PASS_ON;
}

void Popup::terminateWindow()
{
	INFO("send result based on response and sender");
	m_sender(getResult());

	clear();
}

void Popup::clear()
{
	if (m_win != nullptr) {
		// clear all resources
		evas_object_del(m_win);
		m_win = nullptr;
	}

	m_buttonTypeMap.clear();
}

} // namespace Ui
} // namespace Csr
