/*
 *  Copyright (c) 2016 Samsung Electronics Co., Ltd All Rights Reserved
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License
 */
/*
 * @file        popup.h
 * @author      Kyungwook Tak (k.tak@samsung.com)
 * @author      Sangwan Kwon (sangwan.kwon@samsung.com)
 * @version     1.0
 * @brief
 */
#pragma once

#include <Elementary.h>
#include <string>
#include <vector>
#include <memory>

#include <csr-content-screening-types.h>
#include <csr-web-protection-types.h>

#include "common/audit/logger.h"
#include "common/binary-queue.h"
#include "common/types.h"

namespace Csr {
namespace Ui {

/*
 * ------------------------
 * |        title         |
 * |==========box=========|
 * ||  content(header)   ||
 * ||-------subBox-------||
 * |||icon|content(body)|||
 * ||--------------------||
 * ||  -hypertext        ||
 * ||   content(footer)  ||
 * ||-------btnBox-------||
 * |||     button(N)    |||
 * ||--------------------||
 * |======================|
 */

class Popup {
public:
	struct Button {
		Button(const char *text, int type) : text(text), type(type) {}

		const char *text;
		int type;
	};

	Popup();
	virtual ~Popup();

	Popup(const Popup &) = delete;
	Popup &operator=(const Popup &) = delete;
	Popup(Popup &&) = delete;
	Popup &operator=(Popup &&) = delete;

	void reset();

	using Sender = std::function<void(const RawBuffer &)>;
	void setSender(Sender &&sender);

	void setMessage(const std::string &msg) noexcept;
	void setTitle(const std::string &title) noexcept;
	void setHeader(const std::string &header) noexcept;
	void setBody(const std::string &body) noexcept;
	void setFooter(const std::string &footer) noexcept;
	void setIcon(const std::string &path) noexcept;
	void setDetailedUrl(const std::string &url);
	void setButtons(const std::vector<Button> &buttons);

private:
	static void btnClickedCb(void *data, Evas_Object *, void *);
	static void hypertextClickedCb(void *data, Evas_Object *, void *);
	static void rotationChangedCb(void *data, Evas_Object *, void *);
	static Eina_Bool keyDownCb(void *, int, void *);

	std::map<Evas_Object *, int> m_buttonTypeMap;
	void setText(Evas_Object *obj, const std::string &text) noexcept;
	void terminateWindow();
	void clear();

	RawBuffer getResult(void);
	void setDefaultProperties(Evas_Object *obj) noexcept;
	void setRotationToWin(Evas_Object *obj) noexcept;

	Evas_Object *m_win = nullptr;
	Evas_Object *m_popup = nullptr;
	Evas_Object *m_box = nullptr;
	Evas_Object *m_subBox = nullptr;
	Evas_Object *m_btnBox = nullptr;
	Evas_Object *m_header = nullptr;
	Evas_Object *m_body = nullptr;
	Evas_Object *m_footer = nullptr;
	Evas_Object *m_icon = nullptr;
	Evas_Object *m_hypertext = nullptr;

	std::string m_message;
	std::string m_iconPath;
	std::string m_hypertextUrl;

	int m_winW = 0;
	int m_winH = 0;

	Sender m_sender;

	static int response;
};

} // namespace Ui
} // namespace Csr
